package SPVM::Error::NotSupported;

1;

=head1 Name

SPVM::Error::NotSupported - Not Supported Error

=head1 Usage

  use Error::NotSupported;

=head1 Description

C<Error::NotSupported> is the error class that the system doesn't support the feature. The class id is C<3>.
